/*
 * small_flask.css_t
 * ~~~~~~~~~~~~~~~~~
 *
 * :copyright: Copyright 2010 by Armin Ronacher.
 * :license: Flask Design License, see LICENSE for details.
 */

body {
    margin: 0;
    padding: 20px 30px;
}

div.documentwrapper {
    float: none;
    background: white;
}

div.sphinxsidebar {
    display: block;
    float: none;
    width: 102.5%;
    margin: 50px -30px -20px -30px;
    padding: 10px 20px;
    background: #333;
    color: white;
}

div.sphinxsidebar h3, div.sphinxsidebar h4, div.sphinxsidebar p,
div.sphinxsidebar h3 a {
    color: white;
}

div.sphinxsidebar a {
    color: #aaa;
}

div.sphinxsidebar p.logo {
    display: none;
}

div.document {
    width: 100%;
    margin: 0;
}

div.related {
    display: block;
    margin: 0;
    padding: 10px 0 20px 0;
}

div.related ul,
div.related ul li {
    margin: 0;
    padding: 0;
}

div.footer {
    display: none;
}

div.bodywrapper {
    margin: 0;
}

div.body {
    min-height: 0;
    padding: 0;
}

.rtd_doc_footer {
    display: none;
}

.document {
    width: auto;
}

.footer {
	width: auto;
}

.footer {
	width: auto;
}

.github {
    display: none;
}